;;;;;;;;;;;;;;;;;;;;;;;;;;;;;<--OGI-->;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                                                        ;;
;;             Center for Spoken Language Understanding                   ;;
;;        Oregon Graduate Institute of Science & Technology               ;;
;;                         Portland, OR USA                               ;;
;;                        Copyright (c) 2000                              ;;
;;                                                                        ;;
;;      This module is not part of the CSTR/University of Edinburgh       ;;
;;               release of the Festival TTS system.                      ;;
;;                                                                        ;;
;;  In addition to any conditions disclaimers below, please see the file  ;;
;;  "license_ogi_tts.txt" distributed with this software for information  ;;
;;  on usage and redistribution, and for a DISCLAIMER OF ALL WARRANTIES.  ;;
;;                                                                        ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;<--OGI-->;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(set! ogi_debug_as_unitsel nil)

(define (item.alloname seg) 
  (let ((name (item.name seg)))
    (cond
     ((string-equal (item.feat seg "allophone_name") "dx") 
      "dx")
     ((string-equal (item.name seg) "h")
      (if (string-equal (item.feat (item.prev seg) "ph_cvox") "-")
	  "hu"
	  "hv"))
     ((string-matches (item.feat seg "allophone_name") "[ptk]h")
      (item.feat seg "allophone_name"))
     (t
     name))))

(define (triphone seg)
  (set! nseg (item.relation.next seg 'Segment))
  (set! nnseg (item.relation.next nseg 'Segment))
  (set! pseg (item.relation.prev seg 'Segment))

  (cond
   ((and (string-equal (item.feat nseg "name") "&")
	 (and (string-equal (item.feat nnseg "name") "m")
	      (string-equal (item.feat nnseg "ogi_onsetcoda") "coda")))
    (string-append (item.alloname seg) "+m"))
   ((and (string-equal (item.feat nseg "name") "&")
	 (and (string-equal (item.feat nnseg "name") "n")
	      (string-equal (item.feat nnseg "ogi_onsetcoda") "coda")))
    (string-append (item.alloname seg) "+n"))
   ((and (string-equal (item.feat nseg "ph_vc") "+")
	 (string-equal (item.feat nnseg "name") "l")
	 (string-equal (item.feat nnseg "ogi_onsetcoda") "coda"))
    (string-append (item.alloname seg) "+l"))
   ((and (string-equal (item.feat nseg "ph_vc") "+")
	 (string-equal (item.feat nnseg "name") "9r")
	 (string-equal (item.feat nnseg "ogi_onsetcoda") "coda"))
    (string-append (item.alloname seg) "+9r"))
   
   ((and (string-equal (item.feat seg "name") "&")
	 (and (string-equal (item.feat nseg "name") "m")
	      (string-equal (item.feat nseg "ogi_onsetcoda") "coda")))
    (string-append (item.alloname pseg) "+m"))
   ((and (string-equal (item.feat seg "name") "&")
	 (and (string-equal (item.feat nseg "name") "n")
	      (string-equal (item.feat nseg "ogi_onsetcoda") "coda")))
    (string-append (item.alloname pseg) "+n"))
   ((and (string-equal (item.feat seg "ph_vc") "+")
	 (string-equal (item.feat nseg "name") "l")
	 (string-equal (item.feat nseg "ogi_onsetcoda") "coda"))
    (string-append (item.alloname pseg) "+l"))
   ((and (string-equal (item.feat seg "ph_vc") "+")
	 (string-equal (item.feat nseg "name") "9r")
	 (string-equal (item.feat nseg "ogi_onsetcoda") "coda"))
    (string-append (item.alloname pseg) "+9r"))
   (t
    0))
  )

(define (as_OGI_di_unitsel utt)
  "Replace unit selection method defined in ogi_unitsel.scm.
   Strategy: if vowel or glide, determine syllable boundary, 
   check if special case diphone exists" 
  (let ((SelectedUnits) (seglist (utt.relation.items utt "Segment"))
	segL
	segR
	Lnm
	Rnm
	diphL
	diphR
	cand)
    (mapcar
     (lambda (segL)
       (set! segR (item.next segL))
        (if (equal? segR nil)
	   nil    ;; if last segment of utt, don't do anything
	   ;; else 
	   (begin
	     (if (not (string-equal (item.alloname segR) "dx"))
		      (set! Rnm (item.name segR))
		      (set! Rnm (item.alloname segR)))

	     (set! Lnm (item.alloname segL))
	     (set! diphL Lnm)
	     (set! diphR Rnm)
	     (cond
	      ;; consonants in coda
	      ((string-equal (item.feat segL "ogi_onsetcoda") "coda")
	       (set! diphL (string-append Lnm "."))
	       )
	      ;; voice vs unvoiced h
;	      ((string-equal (item.name segL) "h")
;	       (if (string-equal (item.feat (item.prev segL) "ph_cvox") "-")
;		   (set! diphL (string-append Lnm "u"))
;		   (set! diphL (string-append Lnm "v")))
;		   )
;	      ((string-equal (item.name segR) "h")
;	       (if (string-equal (item.feat segL "ph_cvox") "-")
;		   (set! diphR (string-append Rnm "u"))
;		   (set! diphR (string-append Rnm "v")))
;	       )
	      ;; glottal stop
	      ((string-equal (item.name segL) "?")
	       (set! diphL "pau"))
	      )
	     ;; voiced/unvoiced h
;	     ((or (string-equal diphR "hv")
;		  (string-equal diphR "hu"))
;	      (set! diphR "h"))
	     ;; end of cond

	     (set! cand (string-append diphL "-" diphR))

	     (if (not (string-equal (item.feat segL "lisp_triphone") "0"))
		 (set! cand (string-append cand "+" (item.feat segL "lisp_triphone"))))
	     ; exception for diphones starting with pau
;	     (if (and (string-equal diphL "pau")
;		      (not (string-equal (item.feat segR "lisp_triphone") "0")))
;		 (set! cand (string-append cand "+" (item.feat segR "lisp_triphone"))))
;	     (format t "%s\n" cand)
	     
	     (if ogi_debug_as_unitsel	    
		 (format stderr " diphL-diphL= %s-%s\n" diphL diphR))
	     
;	     (set! prevdiphL diphL)
	     
	     ;; look up in dbase
	     (set! indx (OGIdbase.unitsel_name cand))
;	     (format t "%s\n" indx)
	     
	     (if (equal? indx nil)
		 (if ogi_debug_as_unitsel	  
		     (format stderr "FAILED......................... BACK TO %s %s\n" Lnm Rnm)
		     )
		 )

	     ;; change from coda to onset if original not found
	     (if (equal? indx nil)
		 (begin
		   (set! cand (string-append (string-before cand ".") (string-after cand ".")))
		   (set! indx (OGIdbase.unitsel_name cand))))

	     ;; change from onset to coda if original not found
	     (if (equal? indx nil)
		 (begin
		   (set! cand (string-append diphL ".-" diphR))
		   (set! indx (OGIdbase.unitsel_name cand))))

	     ;; look for alternates if original not found
	     (if (equal? indx nil)
		 (set! indx (OGI_di_alt Lnm Rnm)))
	     
	     ;; lookup info structure for this unit
	     (set! sel (list (OGIdbase.lookup_info indx)))
;	     (format t "%l\n" sel)
	     
	     ;; append info to list
	     (set! SelectedUnits (append SelectedUnits sel))
	     )))
     seglist)
    SelectedUnits
    )
  )
